screen = {
	width = love.window.getWidth(),
	height = love.window.getHeight()
}

function love.load()
	--Loading classes
	fmath = require "src/class/fmath"
	state = require "src/class/state"
	camera = require "src/class/camera"
	bump = require "src/class/bump"
	level = require "src/class/levelHandler"
	tile = require "src/class/tile"
	physics = require "src/class/physics"
	entity = require "src/class/entity"
	controls = require "src/class/controls"

	--Löve setup
	love.graphics.setDefaultFilter("nearest", "nearest")
	love.graphics.setLineStyle("rough")
	love.graphics.setBackgroundColor(29, 29, 29)

	--Global variables
	resolution = love.window.getFullscreenModes()
	table.sort(resolution, function(a, b) return a.width*a.height < b.width*b.height end)
	currentResolution = 7
	resize(resolution[currentResolution].width, resolution[currentResolution].height)

	smoothFactor = 2 * scale
	smoothCutoff = 0.9

	showAABB = false
	showDebug = true

	love.graphics.setLineWidth(scale)


	resetBump()


	--Loading images
	tileSheet = love.graphics.newImage("src/img/tile.png")
	playerSheet = love.graphics.newImage("src/img/player.png")

	--States
	state:newState("game", "src/state/game")

	--Entities
	entity:new("player", "src/entity/player")
	entity:new("coin", "src/entity/coin")

	--Setup
	tile:load()
	controls:load()
	state:setState("game")
end

function resize(width, height)
	screen.width = width
	screen.height = height
	scale = math.floor(math.min(width, height) * 0.005)
	assetSize = 16
	tileSize = assetSize * scale
	love.window.setMode(width, height, {fullscreen = false})
	physics.gravity = 600 * scale
	physics.maxVel = 800 * scale
end

function resetBump()
	world = bump.newWorld(tileSize)
	world:addResponse("water")
end

function water(world, col, x,y,w,h, goalX, goalY, filter)
  return goalX, goalY, {}, 0
end

function love.update(dt)
	controls:update(dt)
	state:update(dt)
end

function love.draw()
	state:draw()
	--DEBUG STRING
	if showDebug then
		local stat = love.graphics.getStats()
		local bumpItems = world:countItems()
		local s = string.format("fps: %d | drawCalls: %d | memory: %d\nBump: %d", love.timer.getFPS(), stat.drawcalls, stat.texturememory, bumpItems)
		love.graphics.setColor(255, 255, 0, 255)
		love.graphics.printf(s, 12, 12, screen.width, "left")
	end
end

function love.keypressed(key)
	if key == "escape" then love.event.push("quit") end

	if key == "f1" then
		currentResolution = currentResolution + 1
		resize(resolution[currentResolution].width, resolution[currentResolution].height)
	end

	state:keypressed(key)
end

function love.mousepressed(x, y, key)
	state:mousepressed(x, y, key)
end