local controls = {
	keys = {
		moveLeft = {"a", "left"},
		moveRight = {"d", "right"},
		jump = {"w", "up"},
		reset = {"r"}
	},
	gamepad = {
		move = "leftx",
		jump = {"a"},
		reset = {"b"}
	},
	deadZone = 0.2
}

function controls:load()
	self.js = love.joystick.getJoysticks()
	for i,v in ipairs(self.js) do
		if not v:isGamepad() then 
			table.remove(self.js, i)
		end
	end

	if #self.js > 0 then
		self.anyGamepad = true
	else
		self.anyGamepad = false
	end
end

function controls:isDown(key)
	local isDown = false
	if self.keys[key] then
		for i,v in ipairs(self.keys[key]) do
			if love.keyboard.isDown(v) then
				isDown = true
			end
		end
	end
	return isDown
end

function controls:isGamepadDown(key)
	if self.anyGamepad then
		local isDown = false
		if self.gamepad[key] then
			for o,b in ipairs(self.js) do
				for i,v in ipairs(self.gamepad[key]) do
					if b:isGamepadDown(v) then
						isDown = true
					end
				end
			end
		end
		return isDown
	else return false end
end

function controls:getAxis(axis)
	if self.anyGamepad then
		for i,v in ipairs(self.js) do
			local a = v:getGamepadAxis(self.gamepad[axis])
			if math.abs(a) < self.deadZone then a = 0 end
			return a
		end
	else return false end
end

function controls:update(dt)

end

return controls