local entity = {
	type = {},
	entity = {}
}

function entity:new(name, file)
	if self.type[name] then
		error("Entity '"..name.."' already defined")
	else
		self.type[name] = require(file)
	end
end

function entity:spawn(name, x, y, tag, param)
	tag = tag or false
	param = param or {}
	if self.type[name] then
		if tag then
			self.entity[tag] = self.type[name].spawn(x, y, id, param)
		else
			local id = #self.entity + 1
			self.entity[id] = self.type[name].spawn(x, y, id, param)
		end
	else
		error("Entity type '"..name.."' not defined")
	end
end

function entity:destroyAll()
	self.entity = {}
end

function entity:destroy(tag)
	if self.entity[tag] then
		if world:hasItem(self.entity[tag]) then
			world:remove(self.entity[tag])
		end
		self.entity[tag] = nil
	end
end

function entity:update(dt)
	for i,v in pairs(self.entity) do
		v:update(dt)
	end
end

function entity:draw()
	for i,v in pairs(self.entity) do
		v:draw()
	end
end

function entity:getTag(tag)
	if self.entity[tag] then
		return self.entity[tag]
	else return false end
end

return entity