level = {}

function level:textToTable(file)
	if love.filesystem.exists(file) then
		local map = {}
		local str = {}
		local mapWidth = 1
		local mapHeight = 1
		local currentLayer = 1

		for line in love.filesystem.lines(file) do
			if line:sub(1, 9) == "tileswide" then
				mapWidth = tonumber(line:sub(10))
			elseif line:sub(1, 9) == "tileshigh" then
				mapHeight = tonumber(line:sub(10))
			elseif line:sub(1, 5) == "layer" then
				currentLayer = tonumber(line:sub(6)) + 1
				str[currentLayer] = "return {"
			elseif tonumber(line:sub(1, 1)) or tonumber(line:sub(1, 2)) then
				str[currentLayer] = str[currentLayer].."{"..line:sub(1, -2).."},"
			end
		end

		for i,v in ipairs(str) do
			v = v:sub(1, -2)
			v = v.."}"
			local f = loadstring(v)
			map[i] = f()
		end

		return map, mapWidth, mapHeight
	else
		error("'"..file.."' not found")
	end
end


function level:tableToMap(raw, width, height)
	resetBump()
	local map = {}
	local startX, startY = 1, 1
	for i,v in ipairs(raw) do
		map[i] = {}
		for y=1, height do
			map[i][y] = {}
			for x=1, width do
				local t = raw[i][y][x]
				if t < 0 then 
					t = false
				elseif t == 5 then
					startX = x - 1
					startY = y - 1
					t = false
				else
					t = t + 1
				end
				--Properties
				--SOLID
				local solid = false
				if t == 2 or t == 3 then
					solid = true
				end
				--PICKUP
				local pickup = false
				if t == 4 or t == 5 then
					pickup = true
					t = false
				end
				--Blue beam
				local jump = false
				if y > 1 then
					if raw[i][y-1][x] == 6 then
						jump = true
						solid = true
					end
				end

				map[i][y][x] = {
					tile = t,
					gridX = x - 1,
					gridY = y - 1,
					solid = solid,
					jump = jump
				}

				if solid then
					world:add(map[i][y][x], map[i][y][x].gridX * tileSize, map[i][y][x].gridY * tileSize, tileSize, tileSize)
				end

				if pickup then
					entity:spawn("coin", (x - 1) * tileSize, (y - 1) * tileSize, false, {tile = raw[i][y][x] + 1})
				end
			end
		end
	end

	return map, startX, startY
end

function level:loadLevel(id)
	if love.filesystem.exists("src/level/"..id..".txt") then
		local raw, width, height = self:textToTable("src/level/"..id..".txt")
		local map, startX, startY = self:tableToMap(raw, width, height)
		local batch = love.graphics.newSpriteBatch(tileSheet, width * height)

		return {batch = batch, map = map, width = width, height = height, startX = startX, startY = startY}
	else
		error("Map '"..id.."' not found!")
	end
end

function level:updateBatch(lvl)
	local startX, endX, startY, endY = camera:getMapBorders(lvl)
	lvl.batch:bind()
	lvl.batch:clear()
	--for i,v in ipairs(lvl.map) do
		for y=startY, endY do
			for x=startX, endX do
				local t1 = lvl.map[1][y][x]
				local t2 = lvl.map[2][y][x]
				if t1.tile and not t2.tile then
					lvl.batch:add(tile.tile[t1.tile], t1.gridX * tileSize, t1.gridY * tileSize, 0, tileSize / assetSize, tileSize / assetSize)
				end
				if t2.tile then
					lvl.batch:add(tile.tile[t2.tile], t2.gridX * tileSize, t2.gridY * tileSize, 0, tileSize / assetSize, tileSize / assetSize)
				end
				--tile:draw(t.tile, t.gridX * tileSize, t.gridY * tileSize, {top = top, bottom = bottom, left = left, right = right})
			end
		end
	--end
	lvl.batch:unbind()
end

function level:draw(lvl)
	--[[
	local startX, endX, startY, endY = camera:getMapBorders(lvl)
	for i,v in ipairs(lvl.map) do
		for y=startY, endY do
			for x=startX, endX do
				local t = lvl.map[i][y][x]
				
				tile:draw(t.tile, t.gridX * tileSize, t.gridY * tileSize, {top = top, bottom = bottom, left = left, right = right})
			end
		end
	end
	]]
	love.graphics.draw(lvl.batch)
end

--[[
--OUTLINE
				local top, bottom, left, right = false, false, false, false
				if t.tile == 2 then
					if y-1 > 0 and y+1 < lvl.height then
						if lvl.map[i][y-1][x].tile == false or lvl.map[i][y-1][x].tile > 3 then
							top = true
						end
						if lvl.map[i][y+1][x].tile == false or lvl.map[i][y+1][x].tile > 3 then
							bottom = true
						end
					end
					if x-1 > 0 and x+1 < lvl.width then
						if lvl.map[i][y][x-1].tile == false or lvl.map[i][y][x-1].tile > 3 then
							left = true
						end
						if lvl.map[i][y][x+1].tile == false or lvl.map[i][y][x+1].tile > 3 then
							right = true
						end
					end
				end
]]

return level