local state = {states = {}, selectedState = false}

function state:newState(name, file)
	self.states[name] = require(file)
end

function state:getState()
	return self.selectedState
end

function state:setState(state)
	if self.states[state] then
		if self.selectedState then
			if state ~= self.selectedState then 
				if type(self.states[self.selectedState].unload) == "function" then
					self.states[self.selectedState]:unload()
				end
			end
		end
		self.selectedState = state
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].load) == "function" then
				self.states[self.selectedState]:load()
			end
		end
	else
		error("State '"..state.."' not found")
	end
end

function state:update(dt)
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].update) == "function" then
				self.states[self.selectedState]:update(dt)
			end
		end
	end
end

function state:draw()
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].draw) == "function" then
				self.states[self.selectedState]:draw()
			end
		end
	end
end

function state:keypressed(key)
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].keypressed) == "function" then
				self.states[self.selectedState]:keypressed(key)
			end
		end
	end
end

function state:keyreleased(key)
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].keyreleased) == "function" then
				self.states[self.selectedState]:keyreleased(key)
			end
		end
	end
end

function state:textinput(t)
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].textinput) == "function" then
				self.states[self.selectedState]:textinput(t)
			end
		end
	end
end

function state:mousepressed(x, y, b)
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].mousepressed) == "function" then
				self.states[self.selectedState]:mousepressed(x, y, b)
			end
		end
	end
end

function state:mousereleased(x, y, b)
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].mousereleased) == "function" then
				self.states[self.selectedState]:mousereleased(x, y, b)
			end
		end
	end
end

function state:setScale(s)
	for i,v in pairs(self.states) do
		if v.setScale then
			v:setScale(s)
		end
	end
end

function state:quit()
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].quit) == "function" then
				self.states[self.selectedState]:quit()
			end
		end
	end
end

return state