local player = {}
local pm = {__index = player}	

function player.spawn(x, y)
	local p = {
		x = x,
		y = y,
		xVel = 0,
		yVel = 0,
		speed = 150 * scale,
		acceleration = 300 * scale,
		friction = 512 * scale,
		airResistance = 256 * scale,
		onGround = false,
		moving = false,
		width = math.floor(tileSize /  3),
		height = tileSize,
		facing = "right",
		jumpTime = 0.2,
		jumpTick = 0.2,
		jumpHeight = 210 * scale,
		inJumpBeam = false,
		stoppedJumping = false,
		quad = {
			love.graphics.newQuad(0, 0, assetSize, assetSize + 1, playerSheet:getWidth(), playerSheet:getHeight()),
			love.graphics.newQuad(16, 0, assetSize, assetSize + 1, playerSheet:getWidth(), playerSheet:getHeight()),
			love.graphics.newQuad(32, 0, assetSize, assetSize + 1, playerSheet:getWidth(), playerSheet:getHeight()),
			love.graphics.newQuad(48, 0, assetSize, assetSize + 1, playerSheet:getWidth(), playerSheet:getHeight()),
			love.graphics.newQuad(64, 0, assetSize, assetSize + 1, playerSheet:getWidth(), playerSheet:getHeight()),
			love.graphics.newQuad(80, 0, assetSize, assetSize + 1, playerSheet:getWidth(), playerSheet:getHeight()),
			love.graphics.newQuad(96, 0, assetSize, assetSize + 1, playerSheet:getWidth(), playerSheet:getHeight()),
			love.graphics.newQuad(112, 0, assetSize, assetSize + 1, playerSheet:getWidth(), playerSheet:getHeight())
		},
		frame = 1,
		frameRate = 16,
		frameTick = 0
	}

	world:add(p, p.x, p.y, math.floor(p.width), math.floor(p.height))

	return setmetatable(p, pm)
end

function player:update(dt)
	--==[[ MOVEMENT ]]==--
	local moveAxis = controls:getAxis("move")
	local moveLeft = false
	local moveRight = false
	if moveAxis then
		if moveAxis > 0 then moveRight = true elseif moveAxis < 0 then moveLeft = true end
	else moveAxis = 0 end
	if controls:isDown("moveLeft") or moveLeft then
		local a = self.acceleration
		if self.xVel > 0 then a = self.acceleration * 4 end
		self.xVel = self.xVel - a * dt
		if self.xVel < -self.speed then self.xVel = -(self.speed) end
		self.moving = true
		self.facing = "left"
	elseif controls:isDown("moveRight") or moveRight then
		local a = self.acceleration
		if self.xVel < 0 then a = self.acceleration * 4 end
		self.xVel = self.xVel + a * dt
		if self.xVel > self.speed then self.xVel = self.speed end
		self.moving = true
		self.facing = "right"
	else
		local f = self.friction
		if not self.onGround then f = self.airResistance end
		if self.xVel > 0 then
			self.xVel = self.xVel - f * dt
			if self.xVel < 0 then self.xVel = 0 end
		else
			self.xVel = self.xVel + f * dt
			if self.xVel > 0 then self.xVel = 0 end
		end
		self.moving = false
	end

	if controls:isDown("jump") or controls:isGamepadDown("jump") then
		if not self.stoppedJumping then
			self.jumpTick = self.jumpTick - dt
			if self.jumpTick > 0 then
				self.yVel = -self.jumpHeight 
			end
		end
	else
		self.stoppedJumping = true
	end

	--==[[ GRAVITY ]]==--
	self.yVel = self.yVel + physics.gravity * dt
	if self.yVel > physics.maxVel then self.yVel = physics.maxVel end

	local nx = self.x + self.xVel * dt
	local ny = self.y + self.yVel * dt

	local finalX, finalY, col, len = world:move(self, nx, ny, self.collisionFilter)

	if len > 0 then
		for i,v in ipairs(col) do
			if v.type == "slide" then
				if v.normal.x ~= 0 then
					self.xVel = 0 
				end
				if v.normal.y ~= 0 then
					self.yVel = 0
				end

				if v.normal.y == -1 then
					if v.other.jump then
						self.yVel = -(self.jumpHeight * 2)
					else
						self.onGround = true
						self.jumpTick = self.jumpTime
						self.stoppedJumping = false
					end
				elseif v.normal.y == 1 then
					self.jumpTick = 0
				end
			elseif v.type == "cross" then
				if v.other.pickup then
					v.other.collected = true
				end
			end
		end
	else
		self.onGround = false
	end

	self.x = finalX
	self.y = finalY

	--==[[ ANIMATION ]]==--
	if self.moving then
		if self.onGround then
			self.frameTick = self.frameTick + dt
			if self.frameTick > (1 / self.frameRate) then
				self.frame = self.frame + 1
				if self.frame > 4 then self.frame = 1 end
				self.frameTick = 0
			end	
		else
			self.frame = 2
		end
	else
		self.frame = 1
		self.frameTick = 0
	end
end

function player.collisionFilter(item, other)
	if other.pickup then
		return "cross"
	else return "slide" end
end

function player:draw()
	local frm = self.frame
	if self.facing == "left" then
		frm = frm + 4
	end
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(playerSheet, self.quad[frm], self.x - self.width, self.y - scale, 0, (self.height) / (assetSize), (self.height) / assetSize)

	--love.graphics.setColor(255, 0, 0)
	--love.graphics.print(tostring(self.inJumpBeam), self.x, self.y - 24)
end

return player