local game = {}
	
function game:load()
	self.resetDelay = 0.5
	self.resetTick = 0.5

	game:reset()
end

function game:reset()
	entity:destroyAll()
	self.level = level:loadLevel(1)
	entity:spawn("player", self.level.startX * tileSize, self.level.startY * tileSize, "player")

	--Checking level
	self.levelWidth = #self.level.map[1][1] * tileSize
	self.levelWiderThanScreen = false
	if self.levelWidth > screen.width then
		self.levelWiderThanScreen = true
	end

	self.levelHeight = #self.level.map[1] * tileSize
	self.levelHigherThanScreen = false
	if self.levelHeight > screen.height then
		self.levelHigherThanScreen = true
	end

	game:setCamera(true, "player")

	self.resetTick = 0.5
end

function game:setCamera(real, ent)
	local e = entity:getTag(ent)
	if e then
		local x = e.x
		local y = e.y
		if self.levelWiderThanScreen then
			x = e.x - (screen.width / 2) + (e.width / 2)
			if x < 0 then
				x = 0 
			elseif x + screen.width > self.levelWidth then
				x = self.levelWidth - screen.width

			end
		else
			x = -((screen.width / 2 ) - (self.levelWidth / 2))
		end

		if self.levelHigherThanScreen then
			y = e.y - (screen.height / 2) + (e.height / 2)
			if y < 0 then
				y = 0 
			elseif y + screen.height > self.levelHeight then
				y = self.levelHeight - screen.height

			end
		else
			y = -((screen.height / 2 ) - (self.levelHeight / 2))
		end

		if real then
			camera:setRealPosition(x, y)
		else
			camera:setPosition(x, y)
		end
	end
end

function game:update(dt)
	self.resetTick = self.resetTick - dt 
	if self.resetTick < 0 then self.resetTick = 0 end

	entity:update(dt)

	game:setCamera(false, "player")

	camera:update(dt)

	if controls:isGamepadDown("reset") then
		if self.resetTick == 0 then
			game:reset()
		end
	end

	level:updateBatch(self.level)
end

function game:draw()
	camera:set()
	love.graphics.setColor(255, 255, 255)
	level:draw(self.level)
	entity:draw()

	if showAABB then
		local items, len = world:getItems()
		love.graphics.print(len, 700, 12)
		if len > 0 then
			love.graphics.setColor(255, 0, 0)
			love.graphics.setLineWidth(1)
			for i,v in ipairs(items) do
				local x, y, w, h = world:getRect(v)
				love.graphics.rectangle("line", x, y, w, h)
			end
		end
	end
	camera:unset()
end

function game:keypressed(key)
	--if key == "r" then
	if controls:isDown("reset") then
		self:reset()
	end
end

return game